//------------------------------------------------------------------------------
//  Author:    	Thierry FRANZETTI
//  Comment:	MXF_TK: This example illustrates op2b mxf file creation.
//
// Copyright (c) 2008 OPENCUBE. All rights reserved.
// -----------------------------------------------------------------------------
#if defined(BC6)	// Borland C Builder 6.0
#include <vcl.h>
#endif

#include <mxf_tk/mxf_tk.hpp>
#include <mxf_tk/I_concrete_material.hpp>
#include <mxf_tk/I_op2b_file.hpp>

#include <iostream>
#include <string>

using namespace opencube::MXFTK_NAMESPACE;
using namespace std;

// MxfTk error handler
I_mxf_error_handler* err_h = NULL;

//Throws an exception when an error occured 
void throw_on_error(); 

//Called if an error occured.
void display_errors();

int main(int argc, char* argv[])
{
  if (argc < 13 )
  {
    cout << "usage : out.mxf out.xml \"ganged\" source_file \"package\" source_file [...source_file] \"package\" [...]" << endl;
    cout << "for instance out.mxf out.xml ganged file1.dv file2.wav package file3.aes package ganged file4.m2v file5.aes package file6.wav package" << endl;
    cout << "will produce an op2b file with 4 concrete materials [file1.dv, file2.wav], [file3.aes]," << endl;
    cout << "[file4.m2v, file5.aes] and [file6.wav]. Thanks to the ganged attributes the resulting generic" << endl;
    cout << "material will contain three tracks:" << endl;
    cout << "Track 1 : file1.dv + file4.m2v" << endl;
    cout << "Track 2 : file2.wav + file5.aes" << endl;
    cout << "Track 3 : file3.aes + file6.wav" << endl;
    exit(0);
  }

  // the op2b mxf file we are going to write.
  I_op2b_file* pi_op2b_file = NULL;
  // Path to the source files to be wrapped
  locators* pi_loc = NULL;

  try
  {
    wcout << L"Retrieve MxfTk Error Handler" << endl;
    if (!GetMxfErrorHandlerInterface(&err_h))
      throw 1;

    wcout << L"MxfTk Initialization" << endl;
    InitMXF_TK();
    throw_on_error();

    wcout << L"Create the op1a mxf file interface" << endl;
    NewOp2bFileInterface(&pi_op2b_file, argv[1]);
    throw_on_error();

    // Use this to tell MxfTk to generate index tables for this file
    wcout << L"Activate Index Tables" << endl;
    pi_op2b_file->set_parameter(INDEX_TABLE, "True");

    // you can set this to true if no special
    // editing is required between adjacent source clips
    pi_op2b_file->set_continuous_decoding(false);

    wcout << L"Build source files list" << endl;
    GetEmptyLocatorInterface(&pi_loc);
    throw_on_error();
    for (int i=3; i<argc; i++)
    {
      if (std::string(argv[i])==std::string("ganged"))
      {
        pi_op2b_file->new_multiplexed_material();
        throw_on_error();
      }
      else if (std::string(argv[i])!=std::string("package"))
      {
        pi_loc->add((const char**)&(argv[i]));
      }
      else
      {
        wcout << L"Create source material" << endl;
        I_concrete_material* pi_conc_mat = NULL;
        NewConcreteMaterialInterface(&pi_conc_mat, pi_loc, frame_wrapping);
        throw_on_error();
        wcout << L"Attach material to the mxf file" << endl;
        pi_op2b_file->add_material(pi_conc_mat);
        throw_on_error();
        FreeLocatorInterface(&pi_loc);
        GetEmptyLocatorInterface(&pi_loc);
      }
    }

    wcout << L"Write mxf file on disk" << endl;
    pi_op2b_file->flush();
    throw_on_error();

    wcout << L"Wait termination" << endl;
    pi_op2b_file->wait_end_flush_thread();
    throw_on_error();

    wcout << L"Output header metadata XML file" << endl; // optional!
    pi_op2b_file->output_xml(argv[2]);
    throw_on_error();
  }
  catch (int)
  {
    wcout << "An error occured!" << std::endl;
    // Retrieving details of the error(s) and display it(them).
    display_errors();
  }

  wcout << L"Close Process" << endl;
  if (pi_op2b_file)
    FreeOp2bFileInterface(&pi_op2b_file);
  if (pi_loc)
    FreeLocatorInterface(&pi_loc);

  return 0;
}

void throw_on_error()
{
  if (err_h->has_an_error())
    throw 1;
}

void display_errors()
{
  // Retrieve error handler
  if (!err_h)
    return;

  // Retrieve list of errors
  error_list* err_l = err_h->get_errors();
  if (err_l)
  {
    err_l->begin();
    I_mxf_error** ppi_mxf_error = NULL;
    while ((ppi_mxf_error = err_l->next()) != NULL) //go through the list
    {
      I_mxf_error* pi_error = *ppi_mxf_error;
      // and display each error 
      wcout << L"Error " << (int)pi_error->get_id() << L": " << pi_error->get_error_msg() << endl;
    }
    // free error list interface
    err_h->free_errors(err_l);
  }
}
