//------------------------------------------------------------------------------
//  Author:    	Thierry FRANZETTI
//  Comment:	MXF_TK: This example illustrates op2b mxf file creation.
//              with external reference to pre-existing mxf files.
//              The resulting Op2b file will not include any essence data
//              siimply references to the external mxf files.
//
// Copyright (c) 2008 OPENCUBE. All rights reserved.
// -----------------------------------------------------------------------------
#if defined(BC6)	// Borland C Builder 6.0
#include <vcl.h>
#endif

#include <mxf_tk/mxf_tk.hpp>
#include <mxf_tk/I_concrete_material.hpp>
#include <mxf_tk/I_op2b_file.hpp>

#include <iostream>
#include <string>
#include <vector>

using namespace opencube::MXFTK_NAMESPACE;
using namespace std;

// MxfTk error handler
I_mxf_error_handler* err_h = NULL;

//Throws an exception when an error occured 
void throw_on_error(); 

//Called if an error occured.
void display_errors();

int main(int argc, char* argv[])
{
  if (argc < 8 )
  {
    cout << "usage : out.mxf \"multiplex\" in1.mxf in2.mxf [in.mxf]...\" [...]" << endl;
    exit(0);
  }

  // the op2b mxf file we are going to write.
  I_op2b_file* pi_op2b_file = NULL;
  std::vector<I_mxf_file*> mxf_files;

  try
  {
    wcout << L"Retrieve MxfTk Error Handler" << endl;
    if (!GetMxfErrorHandlerInterface(&err_h))
      throw 1;

    wcout << L"MxfTk Initialization" << endl;
    InitMXF_TK();
    throw_on_error();

    wcout << L"Create the op1a mxf file interface" << endl;
    NewOp2bFileInterface(&pi_op2b_file, argv[1]);
    throw_on_error();

    for (int i=2; i<argc; i++)
    {
      if (std::string(argv[i])==std::string("multiplex"))
      {
        pi_op2b_file->new_multiplexed_material();
        throw_on_error();
      }
      else
      {
        I_mxf_file* pi_mxf_file = NULL;
        wcout << L"Loading " << argv[i] << endl;
        NewMxfFileInterface(&pi_mxf_file, argv[i]);
        throw_on_error();

        mxf_files.push_back(pi_mxf_file);

        concrete_list* embedded_mat = pi_mxf_file->embedded_material();
        if (embedded_mat && embedded_mat->size()>0)
        {
          wcout << L"Top-Level Source Packages (embedded material): " << (long)embedded_mat->size() << endl;
          // get the first concrete material from the list.
          I_concrete_material** ppi_conc_mat = embedded_mat->next();
          if (ppi_conc_mat!=NULL)
          {
            I_concrete_material* pi_conc_mat = *ppi_conc_mat;
            if (pi_conc_mat)
            {
              wcout << L"Attach material to the mxf file" << endl;
              size_t s = std::string(argv[i]).size();
              wchar_t* wp = new wchar_t[s+1];
              iso7_to_utf16(wp, argv[i], int(s));
              wp[s] = wchar_t(0);
              pi_op2b_file->add_external_material(pi_conc_mat, wp);
              delete [] wp;
              throw_on_error();
            }
          }
          pi_mxf_file->free_concrete_list(embedded_mat);
        }
        throw_on_error();
      }
    }

    wcout << L"Write mxf file on disk" << endl;
    pi_op2b_file->flush();
    throw_on_error();

    wcout << L"Wait termination" << endl;
    pi_op2b_file->wait_end_flush_thread();
    throw_on_error();
  }
  catch (int)
  {
    wcout << "An error occured!" << std::endl;
    // Retrieving details of the error(s) and display it(them).
    display_errors();
  }

  wcout << L"Close Process" << endl;
  if (pi_op2b_file)
    FreeOp2bFileInterface(&pi_op2b_file);

  for (std::vector<I_mxf_file*>::iterator it = mxf_files.begin();
    it != mxf_files.end();
    it++)
  {
    FreeMxfFileInterface(&(*it));
  }

  return 0;
}

void throw_on_error()
{
  if (err_h->has_an_error())
    throw 1;
}

void display_errors()
{
  // Retrieve error handler
  if (!err_h)
    return;

  // Retrieve list of errors
  error_list* err_l = err_h->get_errors();
  if (err_l)
  {
    err_l->begin();
    I_mxf_error** ppi_mxf_error = NULL;
    while ((ppi_mxf_error = err_l->next()) != NULL) //go through the list
    {
      I_mxf_error* pi_error = *ppi_mxf_error;
      // and display each error 
      wcout << L"Error " << (int)pi_error->get_id() << L": " << pi_error->get_error_msg() << endl;
    }
    // free error list interface
    err_h->free_errors(err_l);
  }
}
