//------------------------------------------------------------------------------
//  Author:    	MXF Team
//  Comment:	MXF_TK: This class defines the xdcam DCP1 file interface.
//				DCP1 file is conform to Doremi and Dolby
//                              DCP implementation.
//
//  The following convention is used for defining parameters and retrun values:
//      in : input parameter (will be used for computation in the function)
//      out : output parameter (will be modified by the function call)
//      in-out : input parameter that may be modified by the function call.
//      free : MXFTk user is reponsible for the deletion of this parameter
//             unless it is passed as a parameter of another function without 
//             the "free" tag.
//
// Copyright (c) 2008 OPENCUBE. All rights reserved.
// -----------------------------------------------------------------------------
#ifndef OC_I_DCP1_FILE_HPP
#define OC_I_DCP1_FILE_HPP

#include <mxf_tk/mxf_tk.hpp>
#include <mxf_tk/I_opatom_file.hpp>

namespace opencube
{
  namespace MXFTK_NAMESPACE
  {
    /* I_dcp1_file:
     * An interface to create an xdcam dv file.
     */
    class I_dcp1_file : public I_opatom_file
    {
    private:
      virtual bool CALLING_CONVENTION set_external_material(I_concrete_material*, const wchar_t*)=0;
    };

    /*
     * Retrieves an dcp1_file interface.
     * I_dcp1_file** pointer to the newly allocated object
     * const char* path to the xdcam file to be written.
     * Call to flush() will effectively cause the creation of
     * the file.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewDcp1FileInterface(I_dcp1_file** /* out free */, 
                                                                       const char* /* in free */);

    /*
     * Retrieves an dcp1_file interface in streaming mode.
     * I_dcp1_file** pointer to the newly allocated object
     * const char* path to the xdcam file to be written.
     * Call to flush() will effectively cause the creation of
     * the file.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION NewDcp1StreamInterface(I_dcp1_file** /* out free */, 
                                                                         I_output_mxf_stream_task* /* in free */);

    /*
     * Frees an dcp1_file interface.
     * I_dcp1_file** pointer to the object to be freed.
     */
    extern "C" mxf_tk_API bool CALLING_CONVENTION FreeDcp1FileInterface(I_dcp1_file** /* in out */);
  }
}

#endif
